%option main
%x analyse_mailto
%x analyse_http
%x analyse_ftp

/*
  Voir :
  http://www.w3.org/Addressing/URL/5_BNF.html
*/


%{
#define AFFICHER(text) (printf(text ": %s\n",yytext))
#define AUTREAFFICHE(text) (printf(text ":\n%s\n",yytext))
#define CHECK(var) (printf("var : %d\n",var))
#define EXIT_ANALYSE() {if(len==0){printf("\n");BEGIN(INITIAL);}}

int len=0;
%}

alpha [a-zA-Z]
digit [0-9]
safe  [$\-_@.&+-] 
extra [!*"'(),]
    // " comment here 
hex {digit}|[a-eA-E]
escape %{hex}{hex} 

xalpha    {alpha}|{digit}|{safe}|{extra}|{escape}
xalphas {xalpha}+
xpalpha {xalpha}|\+
xpalphas {xpalpha}+

segment {xpalphas}
path {segment}?(\/{segment})*

port {digit}{1,4}
hostname {digit}{1,3}\.{digit}{1,3}\.{digit}{1,3}\.{digit}{1,3}|{xpalphas}
user {xalphas}
login {user}@{hostname}
hostport {hostname}(:{port})?
search \?{xalphas}

http "http://"
ftp  "ftp://"
mailto "mailto:"

httpaddress {http}{hostport}(\/{path})?({search})? 
ftpaddress {ftp}{login}\/{path} 
mailtoaddress {mailto}{xalphas}@{hostname} 

%%
{mailtoaddress} {
	AUTREAFFICHE("Protocole mailto");
	len=yyleng;
	yyless(0);
	BEGIN(analyse_mailto);
}

{httpaddress} {
       AUTREAFFICHE("Protocole http");
       len=yyleng;
       yyless(0);
       BEGIN(analyse_http);
       }

{ftpaddress}  {
       AUTREAFFICHE("Protocole ftp");
       len=yyleng;
       yyless(0);
       BEGIN(analyse_ftp);
       }


<analyse_ftp,analyse_http>{

{login} {
	AFFICHER("Login");
	len-=yyleng;
	
	EXIT_ANALYSE();
}


{hostport} {
	   AFFICHER("Hote (et port)");
       	   len-=yyleng;
	   EXIT_ANALYSE();	   
}


{path} {
       AFFICHER("Chemin");
       len-=yyleng;
       EXIT_ANALYSE();	
}

{search} {
	   AFFICHER("Requete");
       	   len-=yyleng;
	   EXIT_ANALYSE();	   
}

}

<analyse_mailto>{
{user} {
       AFFICHER("Utilisateur");
       len-=yyleng;
       EXIT_ANALYSE();
}

{hostname} {
	   AFFICHER("Hote");
       len-=yyleng;
       EXIT_ANALYSE();

	   }

}


<analyse_ftp,analyse_mailto,analyse_http>{

{mailto}|{http}|{ftp} {
	len-=yyleng;
	EXIT_ANALYSE();
}

.|\n {
       len-=yyleng;
       EXIT_ANALYSE();	

}

}


.|\n {}

%%
